#include <boost/bind.hpp>
#include "protocoltimerasio.h"


ProtocolIOAsio::ProtocolIOAsio(boost::asio::io_service & ioservice) : ioservice(ioservice)
{
}


boost::posix_time::ptime ProtocolIOAsio::currentTimeUTC()
{
  return boost::posix_time::microsec_clock::universal_time();
}


TheTimerObjectBase * ProtocolIOAsio::getTimer()
{
  return new TheTimerObjectAsio(ioservice);
}


void TheTimerObjectAsio::timerExpiredEvent(const boost::system::error_code& err)
{
  if (boost::asio::error::operation_aborted==err)
  {
  }
  else
  {
    timerActive=false;
    if (myHandlerBase)
    {
      (*myHandlerBase)();
    }
  }
}


TheTimerObjectAsio::TheTimerObjectAsio(boost::asio::io_service & ioservice) : ioservice(ioservice), timer(ioservice)
{
  timerActive=false;
  myHandlerBase=0;
}


TheTimerObjectAsio::~TheTimerObjectAsio()
{
  stop();
}


void TheTimerObjectAsio::startAlarmAt(const boost::posix_time::ptime & time, TimerEventHandlerBase * handler)
{
  myHandlerBase=handler;
  timer.expires_at(time);
  timer.async_wait(boost::bind(&TheTimerObjectAsio::timerExpiredEvent, this, boost::asio::placeholders::error));
  timerActive=true;
}


void TheTimerObjectAsio::startAlarmAfter(const boost::posix_time::time_duration & expiry_time, TimerEventHandlerBase * handler)
{
  myHandlerBase=handler;
  timer.expires_from_now(expiry_time);
  timer.async_wait(boost::bind(&TheTimerObjectAsio::timerExpiredEvent, this, boost::asio::placeholders::error));
  timerActive=true;
}


void TheTimerObjectAsio::stop()
{
  if (timerActive)
  {
    timer.cancel();
    timerActive=false;
  }
}

bool TheTimerObjectAsio::isRunning()
{
  return timerActive;
}


